const Cache = require('flat-cache')
const os = require('os')
const path = require('path')
const cache = Cache.load(require('../package.json').name, path.join(os.homedir(), '.cache', 'zed-sync'))

const _getKey = cache.getKey.bind(cache)
const _setKey = cache.setKey.bind(cache)
const _removeKey = cache.removeKey.bind(cache)
const _save = cache.save.bind(cache)

let timer
function saveImmediately() {
  timer && clearTimeout(timer)
  timer = setTimeout(cache.save)
}
cache.getRaw = _getKey
cache.get = cache.getKey = key => {
  const val = _getKey(key)
  if (val === void 0 || val === null) return null
  if (val.expire) {
    if (val.expire > Date.now()) {
      // update expire
      cache.set(key, val.value, val.ttl)
      return val.value
    }
    return null
  }
  return val
}

cache.set = cache.setKey = (key, value, ttl) => {
  _setKey(key, {
    value,
    ttl,
    expire: ttl ? Date.now() + ttl : null,
  })
  saveImmediately()
}

cache.remove = cache.removeKey = key => {
  _removeKey(key)
  saveImmediately()
}

cache.save = (noPrune=true) => {
  // clean all expired
  cache.keys().forEach(_key => {
    const val = _getKey(_key)
    if (val && val.expire && val.expire <= Date.now()) {
      cache.removeKey(_key)
    }
  })
  return _save(noPrune)
}

module.exports = cache
