const fs = require('fs')
const colors = require('colors')
const chokidar = require('chokidar')
const utils = require('./utils')
const logger = utils.logger

let originalBranch = '', currentBranch = '', lastCheckoutTime = 0
const headFile = process.cwd() + '/.git/HEAD'

function getBranchFromFile() {
  const head = fs.readFileSync(headFile, 'utf-8').trim()
  if (!head) return ''
  const match = head.match(/\/([^/]*)$/)
  if (match) { // head 也可能是纯数字的 commit id, 这种情况一般是 pull --rebase 之类的操作, 因此返回原branch
    return match[1]
  } else if (head.match(/^\d+$/)) {
    return currentBranch
  }
  return ''
}

module.exports = {
  isChecking() {
    // logger.log('lastCheckoutTime', lastCheckoutTime, Date.now())
    return Date.now() - lastCheckoutTime < 2600
  },
  isOriginalBranch() {
    // logger.log('isOriginalBranch', currentBranch, originalBranch)
    return currentBranch === originalBranch
  },
  getBranchFromFile,
  getCurrentBranch() {
    return currentBranch
  },
  init: function() {

    originalBranch = getBranchFromFile()
    currentBranch = originalBranch
    logger.log('Current Branch is '.green + originalBranch)

    const headWatcher = chokidar.watch(headFile, {
      persistent: true
    }).on('change', function (file) {
      const checkedBranch = getBranchFromFile()
      if (checkedBranch !== currentBranch) {
        currentBranch = checkedBranch
        lastCheckoutTime = Date.now()
        logger.log('Checkout branch '.yellow + currentBranch);
      }
    })
  }
}
